/* The startup chime is played starting at about $408435BC in the ROM.
   Somewhere in there, I can modify the code to jump to my special
   startup chime instead of doing what it currently does.
   This code will be stored in what appears to be blank space from
   $51D70 to $5A82F ($40851D70 to $4085A82F). That's a total of 35,520 bytes.
   In that space there is a repeating copyright notice and date, and some other weird ASCII.
   That makes plenty of room for a nice sampled startup chime of about 1.5 seconds at 22254 Hz
   
   The code that will call this code loads a return address into register a6.
   I have to preserve it and jump to it when finished.
   
   You provide the sampled sound in the following format:
   (4 byte integer, big endian, specifying the number of samples)
   (samples, in 8-bit unsigned format, at 22254 Hz)

   Without further adieu, here's my (commented) custom sampled startup chime code!
*/

.arch 68030

.section .text
.global _start

_start:
SampledChime:
	lea		0x50F14000, %a3								/* Grab the address of the Apple Sound Chip */
	lea		SampledSound, %a0							/* Address of the sampled sound */
	
	/* Initialize sound chip */
	move.b	#0x01, 0x801(%a3)							/* Go into FIFO mode */
	clr.b	0x807(%a3)									/* Sample rate = 22254 Hz */
	move.b	#0x40, 0x806(%a3)							/* Volume control */
	move.l	#0xFE00FE00, 0x830(%a3)						/* Don't know what this does... */
	move.l	#0xFE00FE00, 0x834(%a3)						/* but the Mac II sound driver does it. */
	move.b	#0x80, 0x803(%a3)							/* Clear FIFO */
	clr.b	0x803(%a3)									/* Stop clearing FIFO */
	tst.b	0x804(%a3)									/* Clear FIFO status flags */
	
	/* Now set up the sound we're playing */
	move.l	(%a0)+, %d0									/* Load the length of the sound (4 byte integer) */
	move.l	%d0, %d1									/* Keep a copy in d1 for the special loop... */
	swap	%d1											/* ...but we want the high 2 bytes shifted down */
	bra.s	NextSample									/* Start the loop! */
	
PutSampleInChip:
	btst	#1, 0x804(%a3)								/* Test the FIFO to make sure it's not full. */
	beq.s	DoSample									/* As long as it's not full, skip down below to write the sample. */
WaitHalfEmpty:											/* But if it IS full, we should wait until it empties a bit. */
	btst	#0, 0x804(%a3)								/* The best way to do this is wait for the FIFO to become half empty. */
	beq.s	WaitHalfEmpty								/* So keep waiting until the half-empty bit comes on, then we can continue on. */

DoSample:
	move.b	(%a0)+, %d2									/* Grab the next sample and increment the pointer */
	move.b	%d2, 0x0(%a3)								/* Write the sample into the FIFO */
	
NextSample:
	/* These next two lines have the same effect as doing dbf on a long (4-byte) word.
	   The dbf instruction only operates on a word, so by letting d1 contain the top word,
	   we can iterate correctly on a 4-byte number. The first loop will be the "remainder"
	   that isn't a multiple of 65536 samples. Then, the first dbf will cause d0 to go to -1 (0xFFFF)
	   and we will fall through to the second dbf. This will go back above and do another
	   set of 65536 samples, and we'll continue doing sets of 65536 samples until d1 gets to 0.
	   Genius...I borrowed this awesome technique from the LC III's sampled startup chime ROM code. */
	dbf		%d0, PutSampleInChip
	dbf		%d1, PutSampleInChip

SoundDone:
	/* Just do the initialization again to make sure it's in a good initial state. */
	move.b	#0x01, 0x801(%a3)							/* Go into FIFO mode */
	clr.b	0x807(%a3)									/* Sample rate = 22254 Hz */
	move.b	#0x40, 0x806(%a3)							/* Volume control */
	move.l	#0xFE00FE00, 0x830(%a3)						/* Don't know what this does... */
	move.l	#0xFE00FE00, 0x834(%a3)						/* but the Mac II sound driver does it. */
	move.b	#0x80, 0x803(%a3)							/* Clear FIFO */
	clr.b	0x803(%a3)									/* Stop clearing FIFO */
	tst.b	0x804(%a3)									/* Clear FIFO status flags */
	
AllDone:
	jmp		(%a6)										/* All done, go back to return address that we were supplied with */
	
SampledSound:											/* This is where the sampled sound should be appended -- compile it, then stick your sound 
														   at the end of the generated binary beginning with the length in samples as a 4-byte
														   big-endian integer. Follow that with the 8-bit unsigned samples. */
	
